@@ -1,34 +1,37 @@
 //
-// Created by Gael Aglin on 26/09/2020.
+// Created by Gael Aglin on 15/01/2022.
 //
+
 #ifndef DL85_DEPTHTWOCOMPUTER_H
 #define DL85_DEPTHTWOCOMPUTER_H
 
 #include "rCover.h"
-#include "trie.h"
-#include "query.h"
-#include "query_best.h"
-#include <chrono>
-#include <utility>
+#include "cache_hash_cover.h"
+#include "cache_trie.h"
+#include "nodeDataManager_Cover.h"
+#include "nodeDataManager_Trie.h"
+#include "rCoverFreq.h"
+#include "depthTwoNodeData.h"
 #include <memory>
 
-using namespace std::chrono;
+class Search_base;
+
+Error computeDepthTwo(RCover*, Error, Attributes &, Attribute, const Itemset &, Node*, NodeDataManager*, Error, Cache*, Search_base*, bool = false);
+
 
-TrieNode* computeDepthTwo(RCover*, Error, Array<Attribute>, Attribute, Array<Item>, TrieNode*, Query*, Error, Trie*);
 
 struct TreeTwo{
-    QueryData_Best* root_data;
+    DepthTwo_NodeData* root_data;
 
     TreeTwo(){
-        root_data = new QueryData_Best();
+        root_data = new DepthTwo_NodeData();
     }
 
-    void replaceTree(TreeTwo* cpy){
-        free();
+    void replaceTree(unique_ptr<TreeTwo> cpy){
         root_data = cpy->root_data;
     }
 
-    void free(){
+    void free() const {
         if (root_data == nullptr) return;
 
         if (root_data->left != nullptr){
@@ -42,6 +45,7 @@ struct TreeTwo{
             delete root_data->right->right;
             delete root_data->right;
         }
+
         delete root_data;
     }
 
