@@ -1,3 +1,5 @@
+import html
+
 from sklearn.base import BaseEstimator
 from sklearn.utils.validation import assert_all_finite, check_X_y, check_array, check_is_fitted
 from sklearn.utils.multiclass import unique_labels
@@ -11,27 +13,31 @@ from enum import Enum
 from subprocess import check_call
 
 
-def get_dot_body(treedict, parent=None, left=True):
+def get_dot_body(treedict, parent=None, left=True, feature_names=None, class_names=None):
     gstring = ""
     id = str(uuid.uuid4())
     id = id.replace('-', '_')
 
     if "feat" in treedict.keys():
         feat = treedict["feat"]
+        if feature_names is not None:
+            feat = feature_names[int(feat)]
+            feat = html.escape(feat)
         if parent is None:
             gstring += "node_" + id + " [label=\"{{feat|" + str(feat) + "}}\"];\n"
-            gstring += get_dot_body(treedict["left"], id)
-            gstring += get_dot_body(treedict["right"], id, False)
+            gstring += get_dot_body(treedict["left"], id, True, feature_names, class_names)
+            gstring += get_dot_body(treedict["right"], id, False, feature_names, class_names)
         else:
             gstring += "node_" + id + " [label=\"{{feat|" + str(feat) + "}}\"];\n"
             gstring += "node_" + parent + " -> node_" + id + " [label=" + str(int(left)) + "];\n"
-            gstring += get_dot_body(treedict["left"], id)
-            gstring += get_dot_body(treedict["right"], id, False)
+            gstring += get_dot_body(treedict["left"], id, True, feature_names, class_names)
+            gstring += get_dot_body(treedict["right"], id, False, feature_names, class_names)
     else:
-        val = str(int(treedict["value"])) if treedict["value"] - int(treedict["value"]) == 0 else str(
-            round(treedict["value"], 3))
-        err = str(int(treedict["error"])) if treedict["error"] - int(treedict["error"]) == 0 else str(
-            round(treedict["error"], 2))
+        val = str(int(treedict["value"])) if treedict["value"] - int(treedict["value"]) == 0 else str(round(treedict["value"], 3))
+        if class_names is not None:
+            val = class_names[int(val)]
+            val = html.escape(val)
+        err = str(int(treedict["error"])) if treedict["error"] - int(treedict["error"]) == 0 else str(round(treedict["error"], 2))
         # maxi = max(len(val), len(err))
         # val = val if len(val) == maxi else val + (" " * (maxi - len(val)))
         # err = err if len(err) == maxi else err + (" " * (maxi - len(err)))
@@ -198,41 +204,46 @@ class DL85Predictor(BaseEstimator):
         """
 
         target_is_need = True if y is not None else False
-        opt_func = self.error_function
-        opt_fast_func = self.fast_error_function
-        opt_pred_func = self.error_function
+        # both slow error function and predictor error function receive the tid list as input
+        # the need or not of a labelling function will define whether the error function
+        # is a slow function or a predictor function but at this point we don't know which one
+        # so they are both set to the input error function
+        user_slow_func = self.error_function  # input: tid list, output: label, error
+        user_fast_func = self.fast_error_function  # input: support per class, output: label, error
+        user_pred_func = self.error_function  # input: tid list, output: error
         predict = True
 
         if target_is_need:  # target-needed tasks (eg: classification, regression, etc.)
             # Check that X and y have correct shape and raise ValueError if not
             X, y = check_X_y(X, y, dtype='int32')
-            if self.leaf_value_function is None:
-                opt_pred_func = None
-                predict = False
-            else:
-                opt_func = None
-                opt_fast_func = None
-            # if opt_func is None and opt_pred_func is None:
-            #     print("No optimization criterion defined. Misclassification error is used by default.")
-        else:  # target-less tasks (clustering, etc.)
+        else:
             # Check that X has correct shape and raise ValueError if not
             assert_all_finite(X)
             X = check_array(X, dtype='int32')
-            if self.leaf_value_function is None:
-                opt_pred_func = None
-                predict = False
-            else:
-                opt_func = None
-                opt_fast_func = None
+
+        if self.leaf_value_function is None:
+            # if no labelling function is defined, the error function is a slow function
+            user_pred_func = None
+            predict = False
+        else:
+            # if a labelling function is defined, the error function is a predictor function
+            user_slow_func = None
+            user_fast_func = None
+
+        # if opt_func is None and opt_pred_func is None:
+        #     print("No optimization criterion defined. Misclassification error is used by default.")
 
         # sys.path.insert(0, "../../")
         import dl85Optimizer
-        # print(opt_func)
+        # print("user_func: ", user_func)
+        # print("user_fast_func: ", user_fast_func)
+        # print("user_pred_func: ", user_pred_func)
+
         solution_str = dl85Optimizer.solve(data=X,
                                            target=y,
-                                           tec_func_=opt_func,
-                                           sec_func_=opt_fast_func,
-                                           te_func_=opt_pred_func,
+                                           tec_func_=user_slow_func,
+                                           sec_func_=user_fast_func,
+                                           te_func_=user_pred_func,
                                            # exw_func_=self.example_weight_function,
                                            # pred_func_=self.predict_error_function,
                                            max_depth=self.max_depth,
@@ -559,7 +570,7 @@ class DL85Predictor(BaseEstimator):
         recurse(tree)
         return tree
 
-    def export_graphviz(self):
+    def export_graphviz(self, feature_names=None, class_names=None):
         if self.is_fitted_ is False:  # fit method has not been called
             raise NotFittedError("Call fit method first" % {'name': type(self).__name__})
 
@@ -577,7 +588,7 @@ class DL85Predictor(BaseEstimator):
                        "node [shape=record]; \n"
 
         # build the body
-        graph_string += get_dot_body(self.tree_)
+        graph_string += get_dot_body(self.tree_, parent=None, left=True, feature_names=feature_names, class_names=class_names)
 
         # end by the footer
         graph_string += "}"
